package scphEditor;

import java.awt.*;

/**
 * This AWT component is the view of a simple string, e.g., the name
 * of an Identifier, a part of an operator.  The string is considered
 * immutable, therefore this class is not an observer.  However,
 * protected methods are provided to access this string, so subclasses
 * can be written to be observers.
 *
 * <p>The default font is null; i.e., follows the parent container
 * dynamically.
 */

public class AtomView extends Component
{
  private String text;
  private Image printimage;
  private boolean simpleprint;

  /**
   * Creates an AtomView object that displays the given string.
   *
   * @param textOf the given string to be displayed.
   */

  public AtomView(String textOf)
  {
    super();
    text = textOf;
    printimage = null;
  }


  /**
   * Gives the string displayed by this view.
   */

  protected String getText()
  {
    return text;
  }


  /**
   * Changes the string displayed by this view.  And redraws itself,
   * of course.
   *
   * @param newText the new string.
   */

  protected void setText(String newText)
  {
    text = newText;
    printimage = null;
    invalidate();
    // needs to validate() the ancestors
    repaint();
  }

  /** 
   * Returns the display size of this AWT component. Both
   * getMinimumSize() and getPreferredSize() return the same thing.
   *
   * @return display size of this component.
   * @see AtomView#getPreferredSize
   */

  public Dimension getMinimumSize()
  {
    FontMetrics fm = getFontMetrics(getFont());
    return new Dimension(fm.stringWidth(text), fm.getHeight());
    // fm.getAscent() + fm.getDescent());
  }

  /** 
   * Returns the display size of this AWT component. Both
   * getMinimumSize() and getPreferredSize() return the same thing.
   *
   * @return display size of this component.
   * @see AtomView#getMinimumSize
   */

  public Dimension getPreferredSize()
  {
    return getMinimumSize();
  }

  public void print(Graphics g)
  {
    if (simpleprint()) {
      super.print(g);
    }
    else {
      if (printimage == null) {
        Dimension size = getSize();
        printimage = createImage(size.width, size.height);
        Graphics og = printimage.getGraphics();
        og.setClip(0, 0, size.width, size.height);
        paint(og);
        og.dispose();
      }
      g.drawImage(printimage, 0, 0, null);
    }
  }

  private boolean simpleprint()
  {
    for (int i = 0; i < text.length(); i++) {
      if (text.charAt(i) >= '\u0100') return false;
    }
    return true;
  }


  /** 
   * Redraws this AWT component.
   *
   * @param g the graphics context in which the drawing is done.
   */
  public void paint(Graphics g)
  {
    Dimension size = getSize();
    g.setColor(getBackground());
    g.fillRect(0, 0, size.width, size.height);
    g.setColor(getForeground());
    g.setFont(getFont());
    g.drawString(text, 0, g.getFontMetrics().getAscent());
  }
}
