package scphEditor;

import java.util.Vector;
import java.util.Observer;
import java.util.Enumeration;
import java.util.Observable;

/**
 * This class re-implements java.util.Observable so it can be cloned safely.
 * But there can be two meanings for cloning an observable:
 * <ul>
 * <li> The clone has no observers to begin with
 * <li> The clone has all of the observers of the original to
 *      begin with
 * </ul>
 * You can choose which one you want when you call the constructor.
 * (The default is the first meaning: empty list.)  In addition,
 * you can change your mind any time in the program flow with the
 * cloneObserverList() method.
 *
 * <p>This class does not implement java.lang.Cloneable!  Reason: you might
 * not want to support cloning in some of your subclasses.  But this class
 * provides a correctly-written protected clone() method, so your
 * subclasses and the subclasses of your subclasses can rely on it.  In
 * other words, this class does not open up access until you want to,
 * and provides proper support when you do open up access.
 *
 * <p>This class subclasses Observable, so that Observer objects cannot
 * tell the difference.  But all Observable methods accessing the observer
 * list are overriden and re-implemented from scratch.  Only the change
 * methods are retained.
 *
 * @see CloneableObservable#cloneObserverList(boolean)
 * @see java.util.Observable
 */

public class CloneableObservable extends Observable
{
  private boolean cloneObservers;
  private Vector observers;

  /**
   * Default constructor.
   */

  public CloneableObservable()
  {
    this(false);
  }


  /**
   * Constructor specifying the copy policy for the observer list.
   *
   * @param cloneObserverList true if you want a clone to duplicate the
   * observer list of the original; false if you want the observer
   * list of a clone to be empty.
   */

  public CloneableObservable(boolean cloneObserverList)
  {
    super();
    observers = new Vector();
    cloneObservers = cloneObserverList;
  }


  /**
   * Clones this object.
   */

  protected synchronized Object clone()
  {
    CloneableObservable c;
    try {
      c = (CloneableObservable) super.clone();
    }
    catch (CloneNotSupportedException e) {
      // should not happen
      throw new InternalError(e.toString());
    }

    if (cloneObservers) {
        c.observers = (Vector) c.observers.clone();
    }
    else {
      c.observers = new Vector();
    }

    return c;
  }


  /**
   * Gives the observer-list cloning policy.
   */

  public synchronized boolean cloneObserverList()
  {
    return cloneObservers;
  }


  /**
   * Changes the observer-list cloning policy.
   *
   * @param newPolicy true if you want a clone to duplicate the
   * observer list of the original; false if you want the observer
   * list of a clone to be empty.
   *
   * @return the old policy.  (Same semantics as the parameter.)
   */

  public synchronized boolean cloneObserverList(boolean newPolicy)
  {
    boolean r = cloneObservers;
    cloneObservers = newPolicy;
    return r;
  }


  // The following re-invents the Observable.

  /**
   * Adds an observer.  No duplication check!
   *
   * @param o the observer.
   */

  public synchronized void addObserver(Observer o)
  {
    observers.addElement(o);
  }


  /**
   * Deletes an observer.
   *
   * @param o the observer.
   */

  public synchronized void deleteObserver(Observer o)
  {
    observers.removeElement(o);
  }


  /**
   * Clears the observer list.
   */

  public synchronized void deleteObservers()
  {
    observers.removeAllElements();
  }


  /**
   * If this object is marked as changed (hasChanged() returns true),
   * mark itself as unchanged (with clearChanged()),
   * and then notify all of its observers.
   * Observers are notified by calling their update()
   * methods with two parameters: this object, and null.
   *
   * @see CloneableObservable#hasChanged
   * @see CloneableObservable#clearChanged
   * @see java.util.Observer#update
   */

  public void notifyObservers()
  {
    notifyObservers(null);
  }

  /**
   * If this object is marked as changed (hasChanged() returns true),
   * mark itself as unchanged (with clearChanged()),
   * and then notify all of its observers.
   * Observers are notified by calling their update()
   * methods with two parameters: this object, and the parameter arg.
   *
   * @param arg the second parameter to be passed to update() methods of
   * observers.
   *
   * @see CloneableObservable#hasChanged
   * @see CloneableObservable#clearChanged
   * @see java.util.Observer#update
   */

  public void notifyObservers(Object arg)
  {
    // this implementation is potentially thread-unsafe.  But it does
    // not introduce deadlock.  Any modification to make it thread-safe
    // should avoid the deadlock problem, which is subtle.

    if (hasChanged()) {
      clearChanged();
      for (Enumeration i = observers.elements(); i.hasMoreElements(); ) {
        Observer ob = (Observer) i.nextElement();
        ob.update(this, arg);
      }
    }
  }


  /**
   * Gives the number of observers of this object.
   */

  public synchronized int countObservers()
  {
    return observers.size();
  }
}
