package scphEditor;

/**
 * An EmbeddingFixed object encapsulates presentation styles, in AWT
 * terms, of an operator and its operands.  The operator is assumed to
 * have a fixed number of operands, e.g., if-then-else, unary
 * operators.
 */

public class EmbeddingFixed implements IEmbedding
{
  private String[] template;
  private int[] map_c2o;        // index map: component to operand
  private int[] map_o2c;        // index map: operand to component

  /**
   * Creates an EmbeddingFixed object.  Requires a template for the
   * display of the operator.  The template specifies where and what
   * symbols are displayed, and also where operands occur.  Examples:
   *
   * <ul>
   * <li>unary prefix ++ operator: <code>{ "++", null }</code>
   * <li>unary postfix ++ operator: <code>{ null, "++" }</code>
   * <li>tertiary if-then-else operator: <code>{ "if", null, "then",
   *     null, "else", null }</code>
   * </ul>
   *
   * In general, the template is an array of strings: null where
   * operands should occur, non-null strings where operator symbols
   * should occur.  Operands are sequentially placed, e.g., if
   * operand#0 then operand#1 else operand#2.
   *
   * @param templateOf the template that specifies operator symbols
   *        and operand positions.
   */
  public EmbeddingFixed(String[] templateOf)
  {
    template = templateOf;
    buildMap();
  }

  /**
   * Takes the template field, builds map_o2c and map_c2o.
   */
  private void buildMap()
  {
    int arity = 0;
    int[] o2c = new int[template.length]; // will copy to map_o2c
    map_c2o = new int[template.length];

    for (int i = 0; i < template.length; i++) {
      if (template[i] != null) {
        map_c2o[i] = -1;
      }
      else {
        map_c2o[i] = arity;
        o2c[arity] = i;
        arity++;
      }
    }
    map_o2c = new int[arity];
    System.arraycopy(o2c, 0, map_o2c, 0, arity);
  }

  /**
   * Gives a new instance of MLMLayout2, the preferred layout manager
   * here.
   *
   * @see MLMLayout2
   */
  public ExpressionLayoutManager getNewLayout()
  {
    return new MLMLayout2();
  }

  public String symbol(int componentIndex)
  {
    return template[componentIndex];
  }

  public int componentIndex(int operandIndex)
  {
    return map_o2c[operandIndex];
  }

  public int operandIndex(int componentIndex)
  {
    if (map_c2o[componentIndex] == -1)
      throw new ArrayIndexOutOfBoundsException();
    return map_c2o[componentIndex];
  }

  public int componentCount(int operandCount)
  {
    return template.length;
  }
}
