package scphEditor;

import java.util.Vector;
import java.util.Enumeration;

/**
 * This class represents compound expressions.  A compound expression
 * has an operator and a list of subexpressions, i.e., children, each
 * child being an Operand object.  E.g., -x is modelled as an Expression
 * object, with the unary - operator and the child x.
 *
 * <p>The arity of an expression is the same as that of its operator;
 * in case the operator is n-ary, though, the expression becomes
 * binary.  Use the subclass ExpressionNary instead if you want true
 * n-ary services.  For example, 1 + 2 + 3 is better modelled by an
 * ExpressionNary object.
 *
 * <p>The update() method of an Observer receives an
 * Expression.Replacement object as a parameter.
 *
 * @see IOperator
 * @see ExpressionNary
 */
public class Expression extends Operand
{
  /**
   * [Rooting] Operator in this expression.
   */
  protected IOperator op;
  /**
   * List of child operands.
   */
  protected Vector children;
  /**
   * True iff this operand is parenthesized.
   */
  protected boolean paren;
  /**
   * True iff the view should split this expression across lines.
   */
  protected boolean splitted;

  /**
   * Constructs an Expression object with the given operator.  The
   * given operator will become the operator of this compound
   * expression permanently.  As many children as the operator arity
   * specifies are set up, and initialized to null.  In the case of an
   * n-ary operator, 2 children are set up.
   *
   * @param opOf the operator object for this Expression object.
   */
  public Expression(IOperator opOf)
  {
    super();

    op = opOf;
    children = new Vector();
    splitted = false;
    paren = false;

    // now we need empty children
    if (op.arity() != IOperator.nARY) {
      // make as many children as the operator says
      children.ensureCapacity(op.arity());
      for (int i = 0; i < op.arity(); i++) {
        children.addElement(null);
      }
    }
  }

  /**
   * Clones this expression.
   * @return a clone of this expression.
   */
  public Object clone()
  {
    // need to copy deeply
    Expression c = (Expression) super.clone();
    c.children = (Vector) c.children.clone();
    for (int i = 0; i < c.children.size(); i++) {
      Operand e = (Operand) c.children.elementAt(i);
      if (e != null) e = (Operand) e.clone();
      c.children.setElementAt(e, i);
    }
    return c;
  }

  /**
   * Gives the operator of this expression.
   *
   * @return the Operator object of this expression.
   */
  public IOperator getOperator()
  {
    return op;
  }

  /*
   * Adopts the given new child as my child at the given index,
   * replacing the previous child at the same index.
   *
   * @param newChild the new child
   * @param atIndex the index to where newChild goes
   * @exception java.lang.ArrayIndexOutOfBoundsException if atIndex
   *   is an invalid index, i.e., atIndex violates my arity.
   */
  public void setChild(Operand newChild, int atIndex)
  {
    try {
      children.setElementAt(newChild, atIndex);
    }
    catch (ArrayIndexOutOfBoundsException e) {
      e.fillInStackTrace();
      throw e;                  // pretend we threw it
    }
    if (newChild != null) newChild.ensureParen(this);
    setChanged();
//     int[] change = new int[countChildren()];
//     for (int i = 0; i < change.length; i++) {
//       change[i] = i == atIndex ? -1 : i;
//     }
    Replacement change = new Replacement(atIndex, 1, 1);
    notifyObservers(change);
  }

  /**
   * Gives my child at the given index.
   *
   * @param atIndex the index at where a child will be sought.
   * @return the child at the index.
   * @exception java.lang.ArrayIndexOutOfBoundsException if atIndex
   *   is an invalid index, i.e., atIndex violates my arity.
   */
  public Operand getChild(int atIndex)
  {
    try {
      return (Operand) children.elementAt(atIndex);
    }
    catch (ArrayIndexOutOfBoundsException e) {
      e.fillInStackTrace();
      throw e;                  // pretend we threw it
    }
  }

  /**
   * This implementation does not do anything.
   */
  public void addChild(Operand newChild, int atIndex)
  {
  }

  /**
   * This implementation returns null and does not do anything else.
   */
  public Operand delChild(int atIndex)
  {
    return null;
  }


  /**
   * Enumerates all of my children, in ascending order of indexes.
   *
   * @return an Enumeration that iterates over my children in ascending
   *   order of indexes.
   */
  public Enumeration getChildren()
  {
    return children.elements();
  }

  public int countChildren()
  {
    return children.size();
  }

  public void addParen()
  {
    paren = true;
    setChanged();
    notifyObservers();
  }

  protected void deParen()
  {
    paren = false;
    setChanged();
    notifyObservers();
  }

  public boolean getParen()
  {
    return paren;
  }

  public boolean getSplit()
  {
    return splitted;
  }

  public void setSplit(boolean split)
  {
    splitted = split;
    setChanged();
    notifyObservers();
  }

  /**
   * Calls v.visitExpression(this).
   */
  public void acceptVisitor(OperandVisitor v)
  {
    v.visitExpression(this);
  }


  /**
   * A Replacement object represents a substitutional change made to
   * an Expression object.  Essentially, a change involves first
   * deleting some children, then inserting some new children.
   */
  public class Replacement
  {
    /**
     * The range of replacement begins at this index.
     */
    public int at;
    /**
     * The replacement involves deleting these many children.  Note
     * the semantics: delete, then insert.
     */
    public int deleteCount;
    /**
     * The replacement involves inserting these many children.  Note
     * the semantics: delete, then insert.
     */
    public int insertCount;

    /**
     * Creates a Replacement object.
     * @param a becomes at.
     * @param dN becomes deleteCount.
     * @param iN becomes insertCount.
     */
    public Replacement(int a, int dN, int iN)
    {
      at = a;
      deleteCount = dN;
      insertCount = iN;
    }
  }
}
