package scphEditor;

/**
 * This class represents n-ary compound expressions.
 */
public class ExpressionNary extends Expression
{
  /**
   * Constructs an n-ary Expression object with the given operator.
   * The given operator will become the operator of this compound
   * expression permanently.  Two null children are set up initially.
   *
   * @param opOf the operator object for this Expression object.
   */
  public ExpressionNary(IOperator opOf)
  {
    this(opOf, 2);
  }

  /**
   * Constructs an n-ary Expression object with the given operator.
   * The given operator will become the operator of this compound
   * expression permanently.  null children are set up initially
   * as specified by the parameters.
   * 
   * @param opOf the operator object for this Expression object
   * @param n number of initial null children to set up
   */
  public ExpressionNary(IOperator opOf, int n) {
    super(opOf);
    if (opOf.arity() != IOperator.nARY) {
      // throw exception
    }
    children.ensureCapacity(n);
    for (int i = 0; i < n; i++) {
      children.addElement(null);
    }
  }

  public void setChild(Operand newChild, int atIndex)
  {
    super.setChild(newChild, atIndex);
  }

  public void addChild(Operand newChild, int atIndex)
  {
    if (atIndex == -1) atIndex = children.size();
    try {
      children.insertElementAt(newChild, atIndex);
    }
    catch (ArrayIndexOutOfBoundsException e) {
      e.fillInStackTrace();
      throw e;                  // pretend we threw it
    }

    if (newChild != null) newChild.ensureParen(this);
    setChanged();
    notifyObservers(new Replacement(atIndex, 0, 1));
  }

  public Operand delChild(int atIndex)
  {
    Operand oldChild;
    try {
      oldChild = (Operand) children.elementAt(atIndex);
    }
    catch (ArrayIndexOutOfBoundsException e) {
      e.fillInStackTrace();
      throw e;                  // pretend we threw it
    }
    children.removeElementAt(atIndex);
    setChanged();
    Replacement change = new Replacement(atIndex, 1, 0);
    notifyObservers(change);
    return oldChild;
  }

  /**
   * Ungroups at the given index.  In other words, if my operator
   * is associative and the child at the given index has the same
   * operator, then use associativity to expand the child into me.
   *
   * <p>Example: if I am 0 + (x + y) + 2, then ungroup(1) turns me
   * into 0 + x + y + 2.
   *
   * @param atIndex the given index.
   */
  public void ungroup(int atIndex)
  {
    if (children.elementAt(atIndex) == null) return;
    int n = ungroup_impl(atIndex);
    if (n > 0) {
      setChanged();
      notifyObservers(new Replacement(atIndex, 1, n));
    }
  }

  public void group(int atIndex, int n)
  {
    if (op.associative() != IOperator.ASSOC) return;
    if (n < 2) return;
    if (n == countChildren()) return;
    ExpressionNary e = new ExpressionNary(op);
    e.children.removeAllElements();
    for (int j = 0; j < n; j++) {
      Operand o = (Operand) children.elementAt(atIndex);
      children.removeElementAt(atIndex);
      e.addChild(o, -1);
    }
    children.insertElementAt(e, atIndex);
    e.ensureParen(this);
    setChanged();
    notifyObservers(new Replacement(atIndex, n, 1));
  }

  /**
   * The bulk of ungroup().  Helps ungroup() and addChild().  It
   * does the ungrouping, but does not send out change notification.
   * (So that ungroup() and addChild() can decide when to do so.)
   *
   * @param atIndex ungrouping occurs at my child at this index.
   *
   * @return 0 if no ungrouping performed (e.g., operators are
   * different or not associative); otherwise the number of new
   * children in the expansion.
   *
   * @see ExpressionNary#addChild
   * @see ExpressionNary#ungroup
   */
  protected int ungroup_impl(int atIndex)
  {
    Operand child;
    try {
      child = (Operand) children.elementAt(atIndex);
    }
    catch (ArrayIndexOutOfBoundsException e) {
      e.fillInStackTrace();
      throw e;                  // pretend we threw it
    }
    if (op != child.getOperator() || op.associative() != IOperator.ASSOC) {
      return 0;
    }
    else {
      children.removeElementAt(atIndex);
      int count = child.countChildren();
      for (int i = 0; i < count; i++) {
        Operand opnd = child.getChild(i);
        children.insertElementAt(opnd, i + atIndex);
        if (opnd != null) opnd.ensureParen(this);
      }
      return count;
    }
  }

  /**
   * Calls v.visitExpressionNary(this).
   */
  public void acceptVisitor(OperandVisitor v)
  {
    v.visitExpressionNary(this);
  }
}
