package scphEditor;

import java.awt.*;
import java.util.Observer;
import java.util.Observable;
import java.util.Enumeration;

/**
 * This AWT component is the view class for Expression objects.  It
 * displays the expression recursively.  It automatically updates
 * itself whenever the expression changes (child manipulations), by
 * way of the Observable-Observer mechanism.
 *
 * @see Expression
 */
public class ExpressionView extends Container implements Observer, OperandView
{
  /**
   * The expression viewed by me.
   */
  protected Expression myClient;
  /**
   * The Embedding for the Operator of my expression.
   */
  protected IEmbedding myEmbedding;
  /**
   * True iff I am showing parentheses.
   */
  protected boolean paren;
  /**
   * Remembers my control object (event listener), so I can pass it on
   * to children.
   */
  protected OperandControl myControl;

  /**
   * Constructor.
   * @param exOf the Expression object to view.
   * @param c the control object (event listener) for me.
   */
  public ExpressionView(Expression exOf, OperandControl c)
  {
    super();
    myClient = exOf;
    myEmbedding = myClient.getOperator().format();
    ExpressionLayoutManager l = myEmbedding.getNewLayout();
    setLayout(l);
    l.setSplit(myClient.getSplit());
    paren = false;
    myControl = c;
    addMouseListener(c);
    addKeyListener(c);
    makeComponents();
  }

  /**
   * Creates my components - views for child operands as well as
   * operator symbols and parentheses.  This implementation works
   * correctly for n-ary operators and expressions, by calling the
   * most abstract methods possible.
   */
  protected void makeComponents()
  {
    OperandViewFactory f = new OperandViewFactory(myControl);
    int arity = myClient.countChildren();
    int cc = myEmbedding.componentCount(arity);
    Enumeration e = myClient.getChildren();

    for (int i = 0; i < cc; i++) {
      if (myEmbedding.symbol(i) != null) {
        add(new AtomView(myEmbedding.symbol(i)), ExpressionLayoutManager.OP);
      }
      else {
        Operand p = (Operand) e.nextElement();
        add(f.make(p));
      }
    }

    parenthesize();
  }

  /**
   * Helper function.  Modifies my parenthesization according to my
   * Expression object.
   */
  protected void parenthesize()
  {
    if (paren && !myClient.getParen()) {
      remove(getComponentCount() - 1);
      remove(0);
      paren = false;
    }
    else if (!paren && myClient.getParen()) {
      add(new AtomView("("), ExpressionLayoutManager.PAREN, 0);
      add(new AtomView(")"), ExpressionLayoutManager.PAREN, -1);
      paren = true;
    }
  }

  public void addNotify()
  {
    myClient.addObserver(this);
    super.addNotify();
  }

  public void removeNotify()
  {
    myClient.deleteObserver(this);
    super.removeNotify();
  }

  /**
   * Returns the Embedding for the Operator of my Expression.
   */
  public IEmbedding getEmbedding()
  {
    return myEmbedding;
  }

  public Operand getOperand()
  {
    return myClient;
  }

  public OperandVector getOperandVector()
  {
    return myClient;
  }

  public int operandIndex(Component child)
  {
    return myEmbedding.operandIndex(viewIndex(child) - (paren ? 1 : 0));
  }

  public Component getOperandView(int opindex)
  {
    return getComponent(myEmbedding.componentIndex(opindex)
                        + (paren ? 1 : 0));
  }

  /**
   * Helper function.  Replaces the child view object at the given index.
   * @param opindex the index.
   * @param view the new view object.
   * @return the replaced, old view object.
   */
  protected Component setOperandView(int opindex, Component view)
  {
    int viewindex = myEmbedding.componentIndex(opindex) + (paren ? 1 : 0);
    Component old = getComponent(viewindex);
    remove(viewindex);
    add(view, viewindex);
    return old;
  }

  /**
   * Given a child view object, looks up the Component index it
   * belongs to in me.
   * @param child the child view object.
   * @return its Component index in me.
   */
  public int viewIndex(Component child)
  {
    // linear search, yikes
    int n = getComponentCount();
    for (int i = 0; i < n; i++) {
      if (getComponent(i) == child) return i;
    }
    return -1;                  // or throw exception?
  }

  public void update(Observable o, Object arg)
  {
    Expression.Replacement change = (Expression.Replacement) arg;
    if (change != null) {
      OperandViewFactory f = new OperandViewFactory(myControl);
      // assume change.deleteCount == 1 and change.insertCount == 1
      setOperandView(change.at, f.make(myClient.getChild(change.at)));
    }
    parenthesize();
    ((ExpressionLayoutManager) getLayout()).setSplit(myClient.getSplit());
    // may need validation here
    // repaint();
  }

  public void paint(Graphics g)
  {
    Dimension size = getSize();
    g.setColor(getBackground());
    g.fillRect(0, 0, size.width, size.height);
    super.paint(g);
  }
}
