package scphEditor;

/**
 * An IEmbedding object encapsulates presentation formatting data
 * of an operator and its operands.  They include:
 *
 * <ul>
 * <li>component indexes of operands
 * <li>component indexes of operator symbols, and the operator symbols
 *     themselves
 * </ul>
 *
 * E.g., consider the if-then-else operator.
 *
 * <ul>
 * <li>the three operands go to components #1, #3, and #5
 * <li>the operator symbols go to components #0, #2, and #4; they are
 *     "if", "then", and "else".
 * </ul>
 */
public interface IEmbedding {
  /**
   * Returns a new instance of the preferred layout manager.
   */
  public abstract ExpressionLayoutManager getNewLayout();

  /**
   * Maps operand index to component index.  If the operand index is
   * invalid (less than zero, or beyond arity), this method may choose
   * to throw an exception.
   *
   * @param operandIndex the operand index.
   * @return the component index.
   * @exception java.lang.ArrayIndexOutOfBoundsException when it is blatantly
   *   obvious to me that the operand index is invalid.
   */
  public abstract int componentIndex(int operandIndex);

  /**
   * Maps component index to operand index.  If the component index is
   * invalid (less than zero, beyond component count, or no operand at
   * the index), this method may choose to throw an exception.
   *
   * @param componentIndex the component index.
   * @return the operand index.
   * @exception java.lang.ArrayIndexOutOfBoundsException when it is blatantly
   *   obvious to me that the component index is invalid.
   */
  public abstract int operandIndex(int componentIndex);

  /**
   * Gives the operator symbol at the component index, or null if
   * there should be an operand at the component index.  If the
   * component index is invalid (less than zero, or beyond component
   * count), this method may choose to throw an exception.
   *
   * @param componentIndex the component index.
   * @return the operator symbol at that index, or null.
   * @exception java.lang.ArrayIndexOutOfBoundsException when it is blatantly
   *   obvious to me that the component index is invalid.
   */
  public abstract String symbol(int componentIndex);

  /**
   * Gives the component count of this embedding.  In other words, how
   * many components an ExpressionView should prepare to contain, in
   * order to accomodate the operands and the operator symbols.  Since
   * this may (or may not) depend on the actual number of operands,
   * this method requires that information.  Implementors are allowed
   * to garbage-in-garbage-out in case the operand count is invalid,
   * e.g., negative.
   *
   * @param operandCount the actual, present number of operands.
   * @return the number of components needed.
   */
  public abstract int componentCount(int operandCount);
}