package scphEditor;

/**
 * IOperator models an operator. An IOperator object is an immutable object (so
 * sorry, no clone), and should be used as singletons and flyweights. You can
 * query an IOperator object for its properties as specified by this interface.
 * 
 * <p>
 * Some examples:
 * <ul>
 * <li>The addition operator should be associative, commutative, n-ary, and
 * have a precedence lower (larger) than the multiplication operator.
 * <li>The negation operator should have a high (small) precedence, be
 * non-associative, non-commutative, and unary.
 * <li>If you consider a variable or constant as an operator too, it should
 * have a very high precedence (say, above negation), be non-associative,
 * non-commutative, and 0-ary.
 * </ul>
 */
public interface IOperator {
  /**
   * If arity() returns nARY, this operator is an n-ary operator, without an
   * upper limit for the number of operands, though the lower limit is 2.
   * Examples are plus,times, and, or.
   * 
   * @see IOperator#arity
   */
  public final static int nARY = -2;

  /**
   * If associative() returns NONASSOC, this operator is non-associative.
   * 
   * @see IOperator#associative
   */
  public final static int NONASSOC = 0;

  /**
   * If associative() returns LEFT, this operator is left-associative.
   * 
   * @see IOperator#associative
   */
  public final static int LEFT = 1;

  /**
   * If associative() returns RIGHT, this operator is right-associative.
   * 
   * @see IOperator#associative
   */
  public final static int RIGHT = 2;

  /**
   * If associative() returns ASSOC, this operator is (two-way) associative.
   * 
   * @see IOperator#associative
   */
  public final static int ASSOC = 3;

  /**
   * Returns the precedence, as a number, of this operator. Larger number means
   * lower precedence (looser binding), smaller number means higher precedence
   * (tighter binding).
   */
  public int precedence();

  /**
   * Returns the associativity of this operator. Possible values are NONASSOC,
   * LEFT, RIGHT, and ASSOC.
   */
  public int associative();

  /**
   * Returns true iff this operator is commutative.
   */
  public boolean commutative();

  /**
   * Returns the arity of this operator. Possible values are natural numbers (my
   * natural numbers include 0) as well as nARY.
   */
  public int arity();

  /**
   * @return Returns the internal unique id.
   */
  public String id();

  /**
   * 
   * @return the format.
   */
  public IEmbedding format();

  /**
   * @return Returns the names on the prover.
   */
  public String[] proverName();

  /**
   * @return Returns the first name on the prover.
   */
  public String proverName0();
}