package scphEditor;

import java.util.HashMap;

/**
 * This class models terminal expressions, such as variables and constants
 * like x and 10.  An Identifier object has a name, and this is precisely
 * the x or 10 part.  It could also have an alternative name to be used
 * with the theorem prover.
 *
 * <p>It implements Operator, because variables and constants are degenerate
 * cases of functions.
 *
 * <p>It is a subclass of Operand as well.  It is an operand that has no
 * child and an operator - itself.
 *
 * <p>Identifier objects are designed to be flyweights.
 * You can use the lookup method to look up an existing Identifier object
 * by its name.
 */
public class Identifier extends Operand implements IOperator
{
  /**
   * dictionary from name to identifier object
   */
  private static HashMap dictionary = new HashMap();
  private String name;          // e.g., x, 10
  private String pname;         // name known to prover

  /**
   * Looks up an Identifier object from the dictionary.
   * @param name The [prover] name of the desired identifier.
   * @return The corresponding identifier or null.
   */
  public static Identifier lookup(String name)
  {
    Identifier i = (Identifier) dictionary.get(name); 
    if (i == null) {
      i = new Identifier(name);
    }
    return i;
  }

  /**
   * Creates an identifier with the given name.
   * @param nameOf The name.
   */
  protected Identifier(String nameOf)
  {
    this(nameOf, nameOf);
  }

  /**
   * Creates an identifier with the given name and prover name.
   * @param nameOf The name.
   * @param pnameOf The prover name.
   */
  protected Identifier(String nameOf, String pnameOf)
  {
    name = nameOf;
    pname = pnameOf;
    dictionary.put(pname, this);
  }

  /**
   * Changes the name and prover name of this identifier.
   * @param nameOf The new name.
   */
  private void setName(String nameOf)
  {
    dictionary.remove(pname);
    name = nameOf;
    pname = nameOf;
    dictionary.put(pname, this);
    setChanged();
    notifyObservers();
  }

  /**
   * Returns the name of this identifier.
   */
  public String getName()
  {
    return name;
  }

  /**
   * Returns the prover name of this identifier.
   */
  public String getPName()
  {
    return pname;
  }

  /**
   * Returns the precedence of this identifier (as an operator).  Always 0.
   */
  public int precedence()
  {
    return 0;
  }

  /**
   * Returns the associativity of this identifier (as an operator).
   * Always Operator.NONASSOC.
   */
  public int associative()
  {
    return IOperator.NONASSOC;
  }

  /**
   * Returns the commutativity of this identifier (as an operator).
   * Always false.
   */
  public boolean commutative()
  {
    return false;
  }

  /**
   * Returns the arity of this identifier (as an operator). Always 0.
   */
  public int arity()
  {
    return 0;
  }

  /**
   * ``Clones'' this identifier.  Since identifiers are flyweights,
   * there is no cloning; the object itself is returned.
   *
   * @return this
   */
  public Object clone()
  {
    return this;
  }

  /**
   * Gives the operator in this identifier operand, which is the
   * identifier itself.
   *
   * @return this
   */
  public IOperator getOperator()
  {
    return this;
  }

  /**
   * Calls v.visitIdentifier(this).
   */
  public void acceptVisitor(OperandVisitor v)
  {
    v.visitIdentifier(this);
  }

  /* (non-Javadoc)
   * @see scphEditor.IOperator#id()
   */
  public String id() {
    return name;
  }

  /* (non-Javadoc)
   * @see scphEditor.IOperator#format()
   */
  public IEmbedding format() {
    return new EmbeddingFixed(proverName());
  }

  /* (non-Javadoc)
   * @see scphEditor.IOperator#proverName()
   */
  public String[] proverName() {
    return new String[] { pname };
  }

  /* (non-Javadoc)
   * @see scphEditor.IOperator#proverName0()
   */
  public String proverName0() {
    return pname;
  }
}
