package scphEditor;

import java.awt.*;
import java.util.Hashtable;

/**
 * This layout manager provides depth-proportional spacing to
 * components.  If you add() a component with a constraint parameter
 * (any non-null string or object will do), the component will obtain
 * depth-proportional spacing around its left and right.  Otherwise
 * the component does not get the extra space.
 *
 * <p>All of minimumLayoutSize(), preferredLayoutSize(), and
 * maximumLayoutSize() return the same value.
 */
public class MLMLayout2 implements ExpressionLayoutManager
{
  /**
   * Padding space = GLUE * depth.
   */
  private int GLUE = 1;
  /**
   * In splitting mode or not.
   */
  private boolean split;
  /**
   * Remembers which components are operators and parentheses.
   */
  private Hashtable operators;

  /**
   * Default constructor.
   */
  public MLMLayout2()
  {
    split = false;
    operators = new Hashtable();
  }

  public void addLayoutComponent(String s, Component com)
  {
    if (s != null) operators.put(com, s);
  }

  public void addLayoutComponent(Component com, Object con)
  {
    if (con != null) operators.put(com, con);
  }

  public void removeLayoutComponent(Component com)
  {
    operators.remove(com);
  }

  public Dimension minimumLayoutSize(Container con)
  {
    return preferredLayoutSize(con);
  }

  public Dimension preferredLayoutSize(Container con)
  {
    if (split) return preferredLayoutSizeSplit(con);
    else return preferredLayoutSizeFlat(con);
  }

  private Dimension preferredLayoutSizeFlat(Container con)
  {
    int space = findDepth(con) * GLUE;
    Dimension ps = new Dimension(0, 0);
    Component[] coms = con.getComponents();
    for (int i = 0; i < coms.length; i++) {
      if (operators.get(coms[i]) == OP)
        ps.width += space;
      Dimension s = coms[i].getPreferredSize();
      ps.width += s.width;
      if (operators.get(coms[i]) == OP)
        ps.width += space;
      if (ps.height < s.height) ps.height = s.height;
    }
    // if (ps.width > 0) ps.width -= space;
    return ps;
  }

  private Dimension preferredLayoutSizeSplit(Container con)
  {
    Dimension[] sizes = sizeSplit(con);
    return new Dimension(sizes[0].width + sizes[1].width + sizes[2].width,
                         Math.max(sizes[0].height, sizes[1].height));
  }

  private Dimension[] sizeSplit(Container con)
  {
    Dimension optrsize = new Dimension(0, 0);
    Dimension opndsize = new Dimension(0, 0);
    Dimension prntsize = new Dimension(0, 0);
    Component[] coms = con.getComponents();

    for (int i = 0; i < coms.length; i++) {
      Component com = coms[i];
      Dimension s = com.getPreferredSize();

      if (operators.get(com) == OP) {
        optrsize.height += s.height;
        optrsize.width = Math.max(optrsize.width, s.width);
      }
      else if (operators.get(com) == PAREN) {
        prntsize.width += s.width;
      }
      else {
        opndsize.height += s.height;
        opndsize.width = Math.max(opndsize.width, s.width);
      }
    }
    return new Dimension[] { optrsize, opndsize, prntsize };
  }

  public Dimension maximumLayoutSize(Container con)
  {
    return preferredLayoutSize(con);
  }

  public void layoutContainer(Container con)
  {
    if (split) layoutContainerSplit(con);
    else layoutContainerFlat(con);
  }

  private void layoutContainerFlat(Container con)
  {
    int space = findDepth(con) * GLUE;
    int cx = 0, cy = 0;
    Component[] coms = con.getComponents();
    for (int i = 0; i < coms.length; i++) {
      if (operators.get(coms[i]) == OP)
        cx += space;
      Dimension s = coms[i].getPreferredSize();
      coms[i].setSize(s);
      coms[i].setLocation(cx, cy);
      cx += s.width;
      if (operators.get(coms[i]) == OP)
        cx += space;
    }
  }

  private void layoutContainerSplit(Container con)
  {
    Dimension[] sizes = sizeSplit(con);
    Component[] coms = con.getComponents();
    int cx = 0, cy = 0;

    for (int i = 0; i < coms.length; i++) {
      Component com = coms[i];
      Dimension s = com.getPreferredSize();

      com.setSize(s);
      if (operators.get(com) == PAREN) {
        if (i == 0) {
          com.setLocation(0, 0);
          cx += s.width;
        }
        else {
          Dimension cs = con.getSize();
          com.setLocation(cs.width - s.width, cs.height - s.height);
        }
      }
      else if (operators.get(com) == OP) {
        com.setLocation(cx, cy);
      }
      else {
        com.setLocation(cx + sizes[0].width, cy);
        cy += s.height;
      }
    }
  }

  public float getLayoutAlignmentX(Container con)
  {
    return 0.0f;
  }

  public float getLayoutAlignmentY(Container con)
  {
    return 0.0f;
  }

  public void invalidateLayout(Container con)
  {
  }

  /**
   * Recursively computes the depth of the container.
   * <ul>
   * <li>Base case: a container without sub-container has depth 0.
   * <li>Induction: depth is 1 + maximum over depths of sub-containers.
   * </ul>
   *
   * @param con the container.
   * @return depth of the container.
   */
  private int findDepth(Container con)
  {
    Component[] coms = con.getComponents();
    int max = -1;
    for (int i = 0; i < coms.length; i++) {
      if (coms[i] instanceof Container) {
        int d = findDepth((Container)coms[i]);
        if (max < d) max = d;
      }
    }
    return max + 1;
  }

  public void setSplit(boolean s)
  {
    split = s;
  }
}
