package scphEditor;

/**
 * This abstract class models an operand.  An operand may be a terminal
 * expression like x or 10; or a compound expression like x+10.  This
 * abstract class is the superclass of both, defining common services
 * and default implementations; this is the base class in a composite pattern.
 *
 * <p>The common services include child management. A compound expression
 * has a number of child operands, e.g., x and 10 are the child operands
 * in x+10. Each child is referred to by a zero-based index; thus, in x+10,
 * x is child #0 and 10 is child #1. Child management, then, consists of
 * methods for:
 * <ul>
 *     <li>setting (adopting) a new child at an index
 *     <li>getting a child at an index
 *     <li>enumerating all children
 * </ul>
 * Naturally, the default implementations of these methods are for leaf
 * nodes (terminal expressions), for which setting and getting children
 * does not make sense; therefore, they always throw exceptions.
 *
 * <p>A compound expression class must override this and other behaviours
 * and implement them for real. This is done by the Expression class
 * hierarchy, for example.
 *
 * @see Expression
 */
public abstract class Operand extends OperandVector implements Cloneable
{
  /**
   * Default constructor.
   */
  public Operand()
  {
    super();
  }

  /**
   * Clones this operand.
   * @return A clone of this operand
   */
  public Object clone()
  {
    return super.clone();
  }

  /**
   * Gives the operator in this subexpression. Yes, this is supposed to
   * make sense even for terminals like x and 10. The current plan is
   * that a terminal is both an operand and an operator; but there is also
   * a chance that a terminal returns some kind of empty operator.
   *
   * @return the Operator object of this operand/subexpression.
   * @see IOperator
   */
  public abstract IOperator getOperator();

  /**
   * Determines if I need to be parenthesized, assuming I am a child
   * of the given operand.  I need parentheses iff my operator binding
   * is looser than my parent's, up to associativity.  Unary operators
   * are treated as right-associative.
   *
   * @param owner my hypothetical or real parent, or null.
   * @return true iff I need to be parenthesized.
   */
  protected boolean needParen(Operand owner)
  {
    if (owner == null) return false;
    int diff = owner.getOperator().precedence() - getOperator().precedence();
    if (diff < 0) return true;
    if (diff > 0) return false;
    int assoc = owner.getOperator().associative();
    if (assoc == IOperator.LEFT && owner.getChild(0) == this)
      return false;
    if ((assoc == IOperator.RIGHT || owner.getOperator().arity() == 1)
        && owner.getChild(owner.countChildren() - 1) == this)
      return false;
    return true;
  }

  /**
   * Parenthesizes me, if I need to be.  Calls needParen() to
   * determine the need and addParen() to do it.  This method is
   * normally only used by setChild() of compound expression classes.
   *
   * @param owner my hypothetical or real parent, or null.
   * @see Operand#needParen
   * @see Operand#addParen
   * @see OperandVector#setChild
   */
  protected void ensureParen(Operand owner)
  {
    if (needParen(owner)) addParen();
  }

  /**
   * Parenthesizes me.
   *
   * <p>The default implementation is for leaf nodes and does nothing.
   * Compound expressions should override this properly.
   */
  public void addParen()
  {
  }

  /**
   * De-parenthesizes me.  This method should only be called by
   * internal code such as removeParen().
   *
   * <p>The default implementation is for leaf nodes and does nothing.
   * Compound expressions should override this properly.
   *
   * @see Operand#removeParen
   */
  protected void deParen()
  {
  }

  /**
   * De-parenthesizes me, if I do not need parentheses.  Calls
   * needParen() to determine the need and deParen() to do it.
   *
   * @param owner my hypothetical or real parent, or null.
   * @see Operand#needParen
   * @see Operand#deParen
   */
  public void removeParen(OperandVector owner)
  {
    if (!(owner instanceof Operand) || !needParen((Operand)owner)) deParen();
  }

  /**
   * Tells if I am already parenthesized.
   *
   * <p>This default implementation is for leaf nodes.  Compound
   * expressions should override this properly.
   *
   * @return true iff I am parenthesized.  (False for leaf nodes.)
   */
  public boolean getParen()
  {
    return false;
  }


  /**
   * Gets my split status.  In other words, whether or not I should be
   * splitted across the line when displayed by a view, like, <tt> b
   * = a</tt> becomes:
   *
   * <pre>
   *   b
   * = a
   * </pre>
   *
   * Of course, this is just a hint to the view.
   *
   * <p>This default implementation is for leaf nodes, which never splits.
   * Compound expressions should override this properly.
   *
   * @return true iff I should be displayed as splitted.  (False for
   * leaf nodes.)
   */
  public boolean getSplit()
  {
    return false;
  }

  /**
   * Sets my split status.
   *
   * <p>This default implementation is for leaf nodes, which never splits.
   * Compound expressions should override this properly.
   *
   * @param split true if you want me to split, false otherwise.
   */
  public void setSplit(boolean split)
  {
  }

  /**
   * Interface for OperandVisitor objects to visit this operand
   * concretely.  (Visitor pattern.)  Absolutely no traversal in any
   * implementation of this method, i.e., this method by itself does
   * not recurse into children of this operand.
   *
   * <p>Implementations of this method are typically one-liners like:
   * <pre>
   *   v.visitConcreteOperand(this);
   * </pre>
   *
   * @param v the visitor object that wants to visit this operand.
   */
  public abstract void acceptVisitor(OperandVisitor v);
}
