package scphEditor;

import java.util.Enumeration;
import java.util.NoSuchElementException;
import java.io.Serializable;

/**
 * This class models a vector of operands.  Operands and Work are
 * vectors of operands.
 *
 * @see Operand
 * @see Work
 */
public abstract class OperandVector
  extends CloneableObservable
  implements Cloneable, Serializable
{
  public OperandVector()
  {
    super();
  }

  /**
   * Adopts the given new child as my child at the given index,
   * replacing the previous child at the same index.
   *
   * <p>This default
   * implementation is for a leaf node in which there must be no
   * child, so it always throws an exception. A compound expression
   * must re-implement this method sensibly.
   *
   * @param newChild the new child
   * @param atIndex the index to where newChild goes
   * @exception java.lang.ArrayIndexOutOfBoundsException if atIndex
   *   is an invalid index. (For leaf nodes, all indexes are invalid.)
   */
  public void setChild(Operand newChild, int atIndex)
  {
    throw new ArrayIndexOutOfBoundsException("No child exists");
  }

  /**
   * Gives my child at the given index.
   *
   * <p>This default implementation
   * is for a leaf node in which there is no child, so it
   * always throws an exception.  A compound expression must
   * re-implement this method sensibly.
   *
   * @param atIndex the index at where a child will be sought.
   * @return the child at the index. (For leaf nodes, this does not make
   *   sense.)
   * @exception java.lang.ArrayIndexOutOfBoundsException if atIndex
   *   is an invalid index. (For leaf nodes, all indexes are invalid.)
   */
  public Operand getChild(int atIndex)
  {
    throw new ArrayIndexOutOfBoundsException("No child exists");
  }

  public void addChild(Operand newChild, int atIndex)
  {
    throw new ArrayIndexOutOfBoundsException("No child exists");
  }

  public Operand delChild(int atIndex)
  {
    throw new ArrayIndexOutOfBoundsException("No child exists");
  }

  /**
   * Enumerates all of my children, in ascending order of indexes.
   *
   * <p>This default implementation returns an ``empty'' Enumeration,
   * i.e., one that has no element at all, which makes sense for leaf nodes.
   * A compound expression must override this behaviour sensibly.
   *
   * @return an Enumeration that iterates over my children in ascending
   *   order of indexes. (For leaf nodes, returns an ``empty'' Enumeration.)
   */
  public Enumeration getChildren()
  {
    return new Enumeration() {
      public boolean hasMoreElements() { return false; }
      public Object nextElement() {
        throw new NoSuchElementException("No child in Operand");
      }
    };
  }

  /**
   * Gives the actual, present number of my children.
   *
   * <p>This default implementation returns zero, as a leaf node has
   * no child.
   *
   * @return the present number of my children.  (Zero for leaf nodes.)
   */
  public int countChildren()
  {
    return 0;
  }
}
