package scphEditor;

import java.awt.Component;

/**
 * This class is a factory of view objects for Operand objects.  It is
 * mainly a helper of the ExpressionView and ExpressionNaryView
 * classes, when they recursively create views for children.  Just
 * call make() with the operand and you are done.
 *
 * <p>It does its job by being a visitor of concrete Operand objects.
 * Each visit method creates an appropriate view object.  In case the
 * given Operand object is null, an appropriate view object is created
 * without visiting.
 *
 * @see OperandViewFactory#make
 */
public class OperandViewFactory implements OperandVisitor
{
  private Component view;
  private OperandControl control;

  /**
   * Constructor.
   */
  public OperandViewFactory(OperandControl c)
  {
    view = null;
    control = c;
  }

  /**
   * Creates and returns a view object for the given Operand object.
   * It works for null too.
   *
   * @param p the operand.
   * @return the view object.
   */
  public Component make(Operand p)
  {
    if (p == null) {
      makeNullView();
    }
    else {
      p.acceptVisitor(this);
    }
    return view;
  }

  public void visitVariable(Variable v)
  {
    view = new IdentifierView(v, control);
    view.setFont(Workspace.IDFONT);
  }

  /**
   * Visits an identifier and creates an IdentifierView object.
   *
   * @param id the visited identifier.
   */
  public void visitIdentifier(Identifier id)
  {
    view = new IdentifierView(id, control);
  }

  /**
   * Visits an Expression object and creates an ExpressionView object.
   * The code of this method does not explicitly recurse into the
   * children of the expression, but since the constructor of
   * ExpressionView (which does) is called, that recursion is
   * implicit.
   *
   * @param e the visited expression.
   */
  public void visitExpression(Expression e)
  {
    view = new ExpressionView(e, control);
  }

  /**
   * Visits an ExpressionNary object and creates an ExpressionViewNary
   * object.  The code of this method does not explicitly recurse into
   * the children of the expression, but since the constructor of
   * ExpressionNaryView (which does) is called, that recursion is
   * implicit.
   *
   * @param e the visited n-ary expression.
   */
  public void visitExpressionNary(ExpressionNary e)
  {
    view = new ExpressionNaryView(e, control);
  }

  /**
   * Creates a new view object for an empty operand, and stores it for
   * retrieval by make().
   */
  private void makeNullView()
  {
    view = new NullView(control);
  }
}
