package scphEditor;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.StreamTokenizer;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * OperatorStore stores operators and provides various retrievals. It also
 * parses input streams for operator specifications.
 */
public class OperatorStore {
  private static Map byId = new LinkedHashMap();

  private static Map byProverName = new HashMap();

  static {
    // hack. need to move to operators file
    IOperator op;
    op = Identifier.lookup("ok");
    byId.put("ok", op);
    byProverName.put("ok", op);

    op = Identifier.lookup("T");
    byId.put("T", op);
    byProverName.put("T", op);

    op = Identifier.lookup("F");
    byId.put("F", op);
    byProverName.put("F", op);
  }
  // Static class. No instance creation.
  private OperatorStore() {
  }

  /**
   * Retrieve an operator by internal identifier.
   * 
   * @param id
   *          an internal identifier
   * @return the operator with internal identifier id, or null
   */
  public static IOperator getById(String id) {
    return (IOperator) byId.get(id);
  }

  /**
   * Retrieve an operator by a prover-side name.
   * 
   * @param proverName
   *          a prover-side name
   * @return the operator with prover-side name proverName, or null
   */
  public static IOperator getByProverName(String proverName) {
    return (IOperator) byProverName.get(proverName);
  }

  /**
   * Retrieve all operators.
   * 
   * @return a collection containing all operators.
   */
  public static Collection getOperators() {
    return byId.values();
  }

  /**
   * Clear the operator store.
   */
  public static void clear() {
    byId.clear();
    byProverName.clear();
  }

  /**
   * Parses a file (assumes UTF-8 encoding) for operator specificaitons and
   * stores them.
   * 
   * @param filename
   *          the name of the file
   * @throws IOException
   */
  public static void parse(String filename) throws IOException {
    FileInputStream f = new FileInputStream(filename);
    parse(f);
    f.close();
  }

  /**
   * Parses an input stream (assumes UTF-8 encoding) for operator specifications and
   * stores them.
   * 
   * @param s
   *          stream to be parsed
   * @throws IOException
   */
  public static void parse(InputStream s) throws IOException {
    Reader r = new InputStreamReader(s, "UTF-8");
    parse(r);
  }

  /**
   * Parses a reader for operator specifications and stores them. Input format
   * to be determined.
   * 
   * @param r
   *          reader to be parsed
   * @throws IOException
   */
  public static void parse(Reader r) throws IOException {
    StreamTokenizer t = new StreamTokenizer(r);
    // turn off comment
    t.ordinaryChar('/');
    // turn on newline detection
    t.eolIsSignificant(true);

    while (t.nextToken() != StreamTokenizer.TT_EOF) {
      // t.ttype == StreamTokenizer.TT_WORD
      String id = t.sval;

      t.nextToken();
      // t.ttype == StreamTokenizer.TT_NUMBER;
      int arity = (int) t.nval;

      t.nextToken();
      // t.ttype == StreamTokenizer.TT_NUMBER
      int prec = (int) t.nval;

      t.nextToken();
      // t.ttype == StreamTokenizer.TT_WORD
      String a = t.sval.intern();
      int assoc;
      if (a == "LEFT") {
        assoc = 1;
      } else if (a == "RIGHT") {
        assoc = 2;
      } else if (a == "ASSOC") {
        assoc = 3;
      } else {
        assoc = 0;
      }

      t.nextToken();
      // t.ttype == StreamTokenizer.TT_WORD
      boolean comm = "true".equals(t.sval);

      t.nextToken();
      // t.ttype == '"'
      String s = unescape(t.sval);
      IEmbedding format = arity == IOperator.nARY ? new EmbeddingNary(s)
          : parse_format_string(s, arity);

      List pn = new ArrayList();
      while (t.nextToken() != StreamTokenizer.TT_EOL) {
        // t.ttype == '"'
        pn.add(t.sval);
      }
      String[] proverName = new String[pn.size()];
      pn.toArray(proverName);

      // should factor out to a separate method
      IOperator op = new DefaultOperator(id, arity, prec, assoc, comm, format,
          proverName);
      byId.put(id, op);
      for (int i = 0; i < proverName.length; i++) {
        String st = proverName[i];
        if (!byProverName.containsKey(st)) {
          byProverName.put(st, op);
        }
      }
    } // end while
  }

  /**
   * Look for \\uxxxx patterns and convert to 16-bit char.
   * @param u input string
   * @return output string
   */
  private static String unescape(String u) {
    StringBuffer b = new StringBuffer(u.length());
    for (int i = 0; i < u.length(); ++i) {
      if (i+1 < u.length() && u.charAt(i) == '\\' && u.charAt(i+1) == 'u') {
        // System.out.println("Beep!");
        char c = (char) Integer.parseInt(u.substring(i+2, i+6), 16);
        b.append(c);
        i += 5;
      } else {
        b.append(u.charAt(i));
      }
    }
    return b.toString();
  }
  
  private static Pattern fsp = Pattern.compile("_");
  private static IEmbedding parse_format_string(String f, int arity) {
    ArrayList v = new ArrayList();
    if (f.charAt(0) == '_') {
      v.add(null);
      f = f.substring(1);
    }
    Matcher m = fsp.matcher(f);
    int p = 0;
    while (m.find()) {
      v.add(f.substring(p, m.start()));
      v.add(null);
      p = m.end();
    }
    if (p != f.length()) {
      v.add(f.substring(p));
    }
    return new EmbeddingFixed((String[]) v.toArray(new String[v.size()]));
  }
}