package scphEditor;

import java.io.IOException;
import java.util.Observable;
import java.util.Observer;

/**
 * ProverShell runs the theorem prover at the back.
 */
public class ProverShell implements Observer {
  // TODO relativize pathnames
  private static final String proverName = "/home/trebla/scphEditor/interrupter hol -I /home/trebla/scphEditor /home/trebla/scphEditor/start.ML";
  private Shell ps;
  private StringBuffer output = new StringBuffer();
  private boolean ready = false;
  private IPostExecute callback = null;

  private static ProverShell singleton;

  /**
   * IPostExecute is the callback type for asynchronous execution.
   * @see ProverShell#execute(String, IPostExecute)
   */
  public static interface IPostExecute {
    /**
     * Callback method for receiving the output of an execution.
     * @param result output of execution
     */
    public abstract void postExecute(String result);
  }

  public static ProverShell instance()
  {
    if (singleton == null) singleton = new ProverShell();
    return singleton;
  }

  private ProverShell()
  {
    try {
      ps = new Shell(proverName);
      ps.addObserver(this);
      ready = false;
      System.out.print("Please wait: loading a *huge* theorem prover... ");
      ps.start();
      synchronized (output) {
        while (!ready)
          try { output.wait(); } catch (InterruptedException ex) {}
      }
      System.out.println("done loading theorem prover");
    }
    catch (IOException e) {
      ps = null;
      System.out.println("Unable to load theorem prover.  You can edit but not prove.");
      System.out.println(e);
    }
  }

  /**
   * Not intended for client use; called by Shell.
   */
  public void update(Observable o, Object arg)
  {
    String cmd = (String)arg;
    synchronized (output) {
      if (cmd.equals("- ")) {
        ready = true;
        output.notify();
        if (callback != null) {
          callback.postExecute(output.toString());
          callback = null;
        }
      }
      else if (cmd.endsWith("\n- ")) {
        ready = true;
        output.append(cmd.substring(0, cmd.length() - 3));
        output.notify();
        if (callback != null) {
          callback.postExecute(output.toString());
          callback = null;
        }
      }
      else {
        output.append(cmd);
      }
    }
  }

  /**
   * Execute a command synchronously. Blocks until command finishes.
   * @param cmd the command
   * @return output from executing the command
   */
  public String execute(String cmd)
  {
    if (ps == null) return "";
    output.setLength(0);
    ready = false;
    try {
      ps.execute(cmd);
    }
    catch (IOException e) {
      return "";
    }
    synchronized (output) {
      try {
        while (!ready) output.wait();
      }
      catch (InterruptedException e) {}
      return output.toString();
    }
  }

  /**
   * Execute a command asynchronously. Returns immediately. To obtain
   * the output when the command finishes later, pass an instance of
   * IPostExecute as callback.
   * @param cmd the command
   * @param p callback or null
   * @see IPostExecute
   */
  public void execute(String cmd, IPostExecute p)
  {
    if (ps == null) {
      p.postExecute("");
      return;
    }
    output.setLength(0);
    ready = false;
    callback = p;
    try {
      ps.execute(cmd);
    }
    catch (IOException e) {
      p.postExecute("");
      callback = null;
    }
  }

  public void interrupt() {
    if (ps != null) ps.interrupt();
  }

  public static void kill()
  {
    if (singleton != null && singleton.ps != null) {
      try {
        singleton.ps.execute("quit();");
      }
      catch (IOException ex) {
      }
      // if (singleton.ps != null) ps.kill();
    }
  }
}
